#pragma once

typedef int (__stdcall * PluginProc)(LPVOID);
typedef int (__stdcall * APIProc)(int, LPVOID);

typedef struct tagAutorunInterface
{
  int nSize;
  HWND hWnd;
  LPVOID pAdapter;
} AutorunInterface;

typedef struct tagVarInfo
{
  int nSize;
  int Flags;
  LPCWSTR VarName;
  LPWSTR VarValue;
} VarInfo;

 // parameters for plugin functions
typedef struct tagAutorunFuncParams
{
  int nSize;
  LPWSTR Param;
  LPWSTR Value;
  int MaxLen;
} AutorunFuncParams;

typedef struct tagPluginFuncInfo
{
  int nSize;
  int Flags;
  LPCWSTR FuncName;
  LPVOID FuncAddr;
} PluginFuncInfo;

// common record for API functions
typedef struct tagGetData
{
  LPWSTR Text;
  LPWSTR Buffer;
} GetData, ExpVars, ParseNextData;

  // buffer size for functions that receive buffers
const int MAX_BUFFER_SIZE = 4096;

  // Function return values (error codes)
const int AU_RESULT_OK = 0;
const int AU_RESULT_ERR_UNKNOWN = 1;
const int AU_RESULT_VARNOEXIST = 2;
const int AU_RESULT_NOREDEFINECONST = 3;

  // Parser function return values
const int AU_PARSER_NOMOREPARAMS = 0;
const int AU_PARSER_REGULARPARAM = 1;
const int AU_PARSER_NAMEDPARARAM = 2;

  // Flags for set variables proc
const int AU_VAR_CONSTANT = 1;

  // Flags for set functions proc
const int AU_FUNC_RETURNVALUE = 1;
const int AU_FUNC_BOOLEAN =     2;

// Indexes of API functions
const int AU_PROC_ADDFUNC     = 1;
const int AU_PROC_PARSER_NEXT = 2;
const int AU_PROC_GETVAR      = 3;
const int AU_PROC_SETVAR      = 4;
const int AU_PROC_EXPANDVARS  = 5;
const int AU_PROC_SHOWERROR   = 6;

LPVOID Adapter;

int SetVar(LPWSTR AName, LPWSTR AValue, int AFlags)
{
  APIProc Proc = (APIProc)Adapter;

  VarInfo Info;
  Info.nSize    = sizeof(VarInfo);
  Info.Flags    = AFlags;
  Info.VarName  = AName;
  Info.VarValue = AValue;

  return Proc(AU_PROC_SETVAR, &Info);
}

int GetVar(LPWSTR AVarName, LPWSTR Buffer)
{
  APIProc Proc = (APIProc)Adapter;

  VarInfo Info;
  Info.nSize    = sizeof(VarInfo);
  Info.VarName  = AVarName;
  Info.VarValue = Buffer;

  return Proc(AU_PROC_GETVAR, &Info);
}

int AddFunc(LPWSTR AName, LPVOID AAddr, int AFlags)
{
  APIProc Proc = (APIProc)Adapter;

  PluginFuncInfo Info;
  Info.nSize    = sizeof(PluginFuncInfo);
  Info.Flags    = AFlags;
  Info.FuncName = AName;
  Info.FuncAddr = AAddr;

  return Proc(AU_PROC_ADDFUNC, &Info);
}

int ParseNext(LPWSTR AParamStr, LPWSTR ABuffer)
{
  APIProc Proc = (APIProc)Adapter;

  ParseNextData Data;
  Data.Text   = AParamStr;
  Data.Buffer = ABuffer;

  return Proc(AU_PROC_PARSER_NEXT, &Data);
}

int ExpandVars(LPWSTR AText, LPWSTR ABuffer)
{
  APIProc Proc = (APIProc)Adapter;

  ExpVars Data;
  Data.Text   = AText;
  Data.Buffer = ABuffer;

  return Proc(AU_PROC_EXPANDVARS, &Data);
}

int ShowError(LPCWSTR AText)
{
  APIProc Proc = (APIProc)Adapter;

  return Proc(AU_PROC_SHOWERROR, (LPVOID)AText);
}
